// created on 10/02/2005 at 18:46
using Gtk;
using Gdk;
using System.Collections;

public class CheezeBuffer : Gtk.TextBuffer {
	public Gtk.TextTagTable tag_table;
	public ArrayList active_tags;
	public CheezePadWindow parent_window;
	
	public CheezeBuffer (Gtk.TextTagTable tags,CheezePadWindow root_window,ArrayList defaultStyles) : base (tags) {
		tag_table = tags;
		parent_window = root_window;
		active_tags = new ArrayList();
		foreach(TextTag tag in defaultStyles){
			toggle_tag_active(tag.Name);
		}
		System.Console.WriteLine("Loaded Default Style");
		foreach(TextTag tag in active_tags)
			System.Console.WriteLine("tag was {0}",tag.Name);
		InsertText += insert_text;
		MarkSet += set_mark;
		Changed += on_changed;
	}
	public bool tag_exists(string tag_name){
		if(tag_table.Lookup(tag_name) != null){
			return true;
		}
		else{
			return false;
		}
	}

	public void create_new_color_tag(bool fg,string name,Gdk.Color col){
		TextTag tag = new TextTag(name);
		if(fg)
			tag.ForegroundGdk = col;
		else
			tag.BackgroundGdk = col;
		tag_table.Add(tag);
	}

	public void set_tag_active(string tag_name) {
		Gtk.TextTag tag = tag_table.Lookup (tag_name);
		Gtk.TextIter sel_start, sel_end;

		if (GetSelectionBounds (out sel_start, out sel_end)) {
			ApplyTag (tag, sel_start, sel_end);
		}
		else {
			active_tags.Add (tag);
		}
	}

	public void apply_tag_to_line(string tag_name){
		Gtk.TextTag tag = tag_table.Lookup (tag_name);
		
		TextIter start = GetIterAtMark(InsertMark);
		start.BackwardLine();
		start.ForwardToLineEnd();
		TextIter end = GetIterAtMark(InsertMark);
		end.ForwardChar();
		end.ForwardToLineEnd();
		
		toggle_tag_active(tag_name);
		ApplyTag(tag,start,end);
	}
	
	public void remove_tag_from_line(string tag_name){
		Gtk.TextTag tag = tag_table.Lookup (tag_name);
		TextIter start = GetIterAtMark(InsertMark);
		start.BackwardLine();
		start.ForwardToLineEnd();
		start.ForwardChar();
		TextIter end = GetIterAtMark(InsertMark);
		end.ForwardToLineEnd();

		RemoveTag(tag,start,end);
	}
	
	public ArrayList find_tags_by_class(string class_name) {
		System.Text.RegularExpressions.Regex isTag= 
			new System.Text.RegularExpressions.Regex(class_name);
		ArrayList remList = new ArrayList();
		foreach(TextTag tag in active_tags){
			if(isTag.IsMatch(tag.Name)){
				remList.Add(tag);
			}
		}
		return remList;
	}

	public void remove_tag_class(string class_name){
		foreach(TextTag tag in ((ArrayList) find_tags_by_class(class_name)))
			remove_tag_active(tag.Name);
	}

	public void remove_tag_class_from_line(string class_name){
		foreach(TextTag tag in ((ArrayList) find_tags_by_class(class_name))){
			remove_tag_active(tag.Name);
			remove_tag_from_line(tag.Name);
		}
	}

	public void remove_tag_active (string tag_name) {
		Gtk.TextTag tag = tag_table.Lookup (tag_name);
		Gtk.TextIter sel_start, sel_end;

		if (GetSelectionBounds (out sel_start, out sel_end)) {
			RemoveTag(tag,sel_start,sel_end);
		}
		else{
			active_tags.Remove (tag);
		}
	}

	public bool active_tag (string tag_name) {
		Gtk.TextTag tag = tag_table.Lookup (tag_name);
		Gtk.TextIter sel_start, sel_end;
		if (GetSelectionBounds (out sel_start, out sel_end))
			return sel_start.BeginsTag(tag) || sel_start.HasTag(tag);
		else
			return active_tags.Contains(tag);
	}

	public void toggle_tag_active (string tag_name){
		Gtk.TextTag tag = tag_table.Lookup (tag_name);
		Gtk.TextIter sel_start, sel_end;

		if (GetSelectionBounds (out sel_start, out sel_end)) {
			if(sel_start.BeginsTag(tag) || sel_start.HasTag (tag))	
				RemoveTag(tag,sel_start,sel_end);
			else
				ApplyTag(tag,sel_start,sel_end);
		}
		else {
			if (active_tags.Contains (tag))
				active_tags.Remove(tag);
			else
				active_tags.Add (tag);
		}
	}
	
	public void set_mark(object o, MarkSetArgs args){
		if(args.Mark != InsertMark)
			return;
		TextIter start, end;
		if(GetSelectionBounds(out start, out end))
			return;

		active_tags.Clear();
		
		TextIter pos = args.Location;
		pos.BackwardChar();
		if(pos.Tags.Length==0){
			foreach(TextTag tag in CheezeStyleSet.defaultTags){
				active_tags.Add(tag);
			}
		}else{
			foreach (Gtk.TextTag tag in pos.Tags){
				if(!pos.BeginsTag(tag))
					active_tags.Add (tag);
			}
			foreach (Gtk.TextTag tag in pos.GetToggledTags(false)){
				if(!pos.EndsTag(tag))
					active_tags.Add (tag);
			}
		}
	}
		

	public void insert_text(object o, InsertTextArgs args){
		TextIter start = args.Pos;
		if(args.Text.Length == 1) {
			//typing
			
			start.BackwardChars(args.Text.Length);
			foreach (Gtk.TextTag tag in start.Tags) {
				RemoveTag (tag, start, args.Pos);
			}

			foreach (Gtk.TextTag tag in active_tags) {
				ApplyTag (tag, start, args.Pos);
			}
		}
	}
	
	public void on_changed(object o, System.EventArgs args){
		uint id = 1;
		
		parent_window.status.Push(parent_window.status.GetContextId("Word Count and Line Count"),
								  string.Concat(
								                "Chr Cnt:",
								                this.CharCount.ToString(),
								                "    Ln Cnt:",
								                this.LineCount.ToString()));
		
	}
	
	public void delete_selection(){
		Gtk.TextIter sel_start, sel_end;
		if (GetSelectionBounds(out sel_start, out sel_end))
			Delete(ref sel_start,ref sel_end);
		return;
	}	
	
	
			
}
